package perpetualbox;

public class Chunk {

	public static int CHUNK_SIZE = 200;
	
	private Pos pos;
	private int[] terrainData;
	private boolean hasBeenModified = false;
	
	public Chunk(Pos inputPos, boolean shouldGenerate) {
		pos = inputPos;
		terrainData = new int[CHUNK_SIZE * CHUNK_SIZE];
		if (shouldGenerate) {
			generateTerrainData();
		}
	}
	
	public static int betterModulus(int number1, int number2) {
		if (number1 >= 0) {
			return number1 % number2;
		} else {
			int output = number2 - ((-number1) % number2);
			if (output >= number2) {
				output -= number2;
			}
			return output;
		}
	}
	
	public void generateTerrainData() {
		Pos tempPos = new Pos(0, 0);
		Pos tempOffset = new Pos(0, 0);
		while (tempOffset.getY() < CHUNK_SIZE) {
			tempPos.set(pos);
			tempPos.add(tempOffset);
			if (tempPos.getY() < 0) {
				setTerrainData(tempPos, 0);
			} else {
				int tempNumber1 = betterModulus(tempPos.getX(), 2);
				int tempNumber2 = betterModulus(tempPos.getY(), 4);
				if ((tempNumber1 == 0 && tempNumber2 == 0) || (tempNumber1 == 1 && tempNumber2 == 2)) {
					setTerrainData(tempPos, 2);
				} else {
					setTerrainData(tempPos, 1);
				}
			}
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= CHUNK_SIZE) {
				tempOffset.setX(0);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
		hasBeenModified = true;
		Database.persistChunk(this);
	}
	
	public Pos getPos() {
		return pos;
	}
	
	public boolean getHasBeenModified() {
		return hasBeenModified;
	}
	
	public void setHasBeenModified(boolean value) {
		hasBeenModified = value;
	}
	
	public boolean containsPos(Pos inputPos) {
		return (inputPos.getX() >= pos.getX() && inputPos.getX() < pos.getX() + CHUNK_SIZE
				&& inputPos.getY() >= pos.getY() && inputPos.getY() < pos.getY() + CHUNK_SIZE);
	}
	
	private int convertPosToIndex(Pos inputPos) {
		return (inputPos.getX() - pos.getX()) + CHUNK_SIZE * (inputPos.getY() - pos.getY());
	}
	
	public int[] getTerrainData() {
		return terrainData;
	}
	
	public int getTerrainData(Pos inputPos) {
		if (!containsPos(inputPos)) {
			return 0;
		}
		int index = convertPosToIndex(inputPos);
		return terrainData[index];
	}
	
	public void setTerrainData(Pos inputPos, int data) {
		if (!containsPos(inputPos)) {
			return;
		}
		int index = convertPosToIndex(inputPos);
		terrainData[index] = data;
		hasBeenModified = true;
	}
	
}
