package perpetualbox;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class CustomPanel extends JPanel {

	private static World world;
	private static int tileSize = 16;
	private static int spriteSize = 32;
	private static int width;
	private static int height;
	private static int worldViewSize = 34;
	private static Color worldBackgroundColor = new Color(196, 255, 255);
	private static Color textColor = new Color(0, 0, 0);
	private static Font textFont = new Font("Verdana", Font.PLAIN, 16);
	private static Image spriteSheet;
	
	public CustomPanel(World inputWorld) {
		super();
		world = inputWorld;
		width = tileSize * worldViewSize;
		height = tileSize * worldViewSize;
		try {
			spriteSheet = ImageIO.read(getClass().getResource("sprites.png"));
		} catch (IOException e) {
			System.out.println("ERROR: Unable to open sprite sheet.");
		}
	}
	
	public void paint(Graphics graphics) {
		graphics.setColor(worldBackgroundColor);
		graphics.fillRect(0, 0, tileSize * worldViewSize, tileSize * worldViewSize);
		
		Pos tempPlayerPos = world.getPlayer().getPos();
		Pos tempCameraPos = new Pos(tempPlayerPos.getX() - worldViewSize / 2,
				tempPlayerPos.getY() - worldViewSize / 2);
		Pos tempOffset = new Pos(-1, -1);
		Pos tempPos = new Pos(0, 0);
		while (tempOffset.getY() < worldViewSize) {
			tempPos.set(tempCameraPos);
			tempPos.add(tempOffset);
			int tempData = world.getTerrainData(tempPos, true);
			if (tempData == 2) {
				drawSprite(graphics, tempOffset, 4);
			}
			if (tempData == 3) {
				drawSprite(graphics, tempOffset, 0);
			}
			tempOffset.setX(tempOffset.getX() + 1);
			if (tempOffset.getX() >= worldViewSize) {
				tempOffset.setX(-1);
				tempOffset.setY(tempOffset.getY() + 1);
			}
		}
		graphics.setFont(textFont);
		graphics.setColor(textColor);
		graphics.drawString("Position: (" + tempPlayerPos.getX() + ", " + tempPlayerPos.getY() + ")", 10, 20);
		graphics.drawString("Inventory: " + world.getPlayer().getInventory(), 10, 40);
	}
	
	public void drawSprite(Graphics graphics, Pos pos, int which) {
		int tempPosX = (which % 4) * 8;
		int tempPosY = (which / 4) * 8;
		graphics.drawImage(spriteSheet, pos.getX() * tileSize, pos.getY() * tileSize, pos.getX() * tileSize + spriteSize, pos.getY() * tileSize + spriteSize,
				tempPosX, tempPosY, tempPosX + 8, tempPosY + 8, null);
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
}
