package perpetualbox;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;

public class Database {

	private static String saveDataDirectory = "./saveData";
	private static String chunkDirectory = saveDataDirectory + "/chunks";
	private static String playerPath = saveDataDirectory + "/player.txt";
	
	public static void persistWorld(World world) {
		System.out.println("SAVING WORLD.");
		int index = 0;
		while (index < world.getChunkCount()) {
			Chunk tempChunk = world.getChunk(index);
			persistChunk(tempChunk);
			index += 1;
		}
		persistPlayer(world.getPlayer());
	}
	
	public static void persistChunk(Chunk chunk) {
		if (!chunk.getHasBeenModified()) {
			return;
		}
		File tempDirectory = new File(saveDataDirectory);
        if (!tempDirectory.exists()) {
            tempDirectory.mkdir();
        }
		tempDirectory = new File(chunkDirectory);
        if (!tempDirectory.exists()) {
            tempDirectory.mkdir();
        }
		try {
			PrintWriter writer = new PrintWriter(getChunkPath(chunk.getPos()), "UTF-8");
			int[] tempTerrainData = chunk.getTerrainData();
			StringBuilder tempBuilder = new StringBuilder();
			int index = 0;
			while (index < tempTerrainData.length) {
				if (index > 0) {
					tempBuilder.append(" ");
				}
				tempBuilder.append(tempTerrainData[index]);
				index += 1;
			}
			String tempText = tempBuilder.toString();
			writer.println(tempText);
			writer.close();
		} catch (IOException exception) {
			System.out.println("ERROR: Unable to save!");
		}
		chunk.setHasBeenModified(false);
	}
	
	public static void persistPlayer(Player player) {
		File tempDirectory = new File(saveDataDirectory);
        if (!tempDirectory.exists()) {
            tempDirectory.mkdir();
        }
		try {
			PrintWriter writer = new PrintWriter(playerPath, "UTF-8");
			writer.println(player.getPos().getX());
			writer.println(player.getPos().getY());
			writer.println(player.getInventory());
			writer.close();
		} catch (IOException exception) {
			System.out.println("ERROR: Unable to save!");
		}
	}
	
	public static World loadWorld() {
		return new World(loadPlayer());
	}
	
	public static Chunk loadChunk(Pos pos, World world) {
		Chunk output = null;
		String tempPath = getChunkPath(pos);
		File tempFile = new File(tempPath);
		if (tempFile.exists())
		{
			try {
				InputStream tempInputStream;
				BufferedReader tempBufferedReader;
				String tempLine;
				tempInputStream = new FileInputStream(tempPath);
				tempBufferedReader = new BufferedReader(new InputStreamReader(tempInputStream, "UTF-8"));
				tempLine = tempBufferedReader.readLine();
				output = new Chunk(pos, false);
				int[] tempData = output.getTerrainData();
				String[] tempList = tempLine.split(" ");
				Pos tempOffset = new Pos(0, 0);
				int index = 0;
				while (index < tempList.length) {
					String tempText = tempList[index];
					int tempValue = Integer.parseInt(tempText);
					tempData[index] = tempValue;
					if (tempValue == 2) {
						Pos tempPos = pos.copy();
						tempPos.add(tempOffset);
						world.addBoxUpdate(tempPos);
					}
					tempOffset.setX(tempOffset.getX() + 1);
					if (tempOffset.getX() >= Chunk.CHUNK_SIZE) {
						tempOffset.setX(0);
						tempOffset.setY(tempOffset.getY() + 1);
					}
					index += 1;
				}
				tempBufferedReader.close();
			} catch (IOException exception) {
				System.out.println("ERROR: Unable to load save file!");
			}
		} else {
			output = new Chunk(pos, true);
		}
		return output;
	}
	
	public static Player loadPlayer() {
		Player output = null;
		File tempFile = new File(playerPath);
		if (tempFile.exists())
		{
			try {
				InputStream tempInputStream;
				BufferedReader tempBufferedReader;
				String tempLine;
				tempInputStream = new FileInputStream(playerPath);
				tempBufferedReader = new BufferedReader(new InputStreamReader(tempInputStream, "UTF-8"));
				tempLine = tempBufferedReader.readLine();
				int tempPosX = Integer.parseInt(tempLine);
				tempLine = tempBufferedReader.readLine();
				int tempPosY = Integer.parseInt(tempLine);
				output = new Player(new Pos(tempPosX, tempPosY));
				tempLine = tempBufferedReader.readLine();
				int tempInventory = Integer.parseInt(tempLine);
				output.setInventory(tempInventory);
				tempBufferedReader.close();
			} catch (IOException exception) {
				System.out.println("ERROR: Unable to load save file!");
			}
		} else {
			output = new Player(new Pos(0, -2));
		}
		return output;
	}
	
	private static String getChunkPath(Pos pos) {
        String tempName = pos.getX() + "_" + pos.getY() + ".txt";
        return chunkDirectory + "/" + tempName;
    }
	
}
