package perpetualbox;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFrame;
import javax.swing.Timer;

public class Main {

	private static World world;
	private static JFrame frame;
	private static CustomPanel panel;
	private static boolean shiftKeyIsHeld = false;
	
	public static void main(String[] args) {
		//testTerrainDataDistribution();
		world = Database.loadWorld();
		frame = new JFrame("PERPETUAL BOX");
		panel = new CustomPanel(world);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		Insets insets = frame.getInsets();
		frame.setSize(insets.left + insets.right + panel.getWidth(),
				insets.top + insets.bottom + panel.getHeight());
		frame.setContentPane(panel);
		frame.setVisible(true);
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				world.tick();
				panel.repaint();
			}
		};
		Timer timer = new Timer(50, actionListener);
		timer.start();
		KeyListener keyListener = new KeyListener() {
			public void keyPressed(KeyEvent actionEvent) {
				int keyCode = actionEvent.getKeyCode();
				//System.out.println(keyCode);
				// A.
				if (keyCode == 65 || keyCode == 37) {
					playerStartActing(new Pos(-1, 0));
				}
				// D.
				if (keyCode == 68 || keyCode == 39) {
					playerStartActing(new Pos(1, 0));
				}
				// W.
				if (keyCode == 87 || keyCode == 38) {
					playerStartActing(new Pos(0, -1));
				}
				// S.
				if (keyCode == 83 || keyCode == 40) {
					playerStartActing(new Pos(0, 1));
				}
				// Shift.
				if (keyCode == 16) {
					shiftKeyIsHeld = true;
				}
			}
			public void keyReleased(KeyEvent actionEvent) {
				int keyCode = actionEvent.getKeyCode();
				// A.
				if (keyCode == 65 || keyCode == 37) {
					playerStopActing(new Pos(-1, 0));
				}
				// D.
				if (keyCode == 68 || keyCode == 39) {
					playerStopActing(new Pos(1, 0));
				}
				// W.
				if (keyCode == 87 || keyCode == 38) {
					playerStopActing(new Pos(0, -1));
				}
				// S.
				if (keyCode == 83 || keyCode == 40) {
					playerStopActing(new Pos(0, 1));
				}
				// Shift.
				if (keyCode == 16) {
					shiftKeyIsHeld = false;
				}
			}
			public void keyTyped(KeyEvent actionEvent) {
				// Do nothing.
			}
		};
		frame.addKeyListener(keyListener);
		Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
            	System.out.println("EXITING.");
                Database.persistWorld(world);
            }
        });
	}
	
	public static void playerStartActing(Pos offset) {
		if (shiftKeyIsHeld) {
			world.getPlayer().placeOrRemoveBlock(offset);
		} else {
			world.getPlayer().startWalking(offset.getX());
		}
	}
	
	public static void playerStopActing(Pos offset) {
		world.getPlayer().stopWalking(offset.getX());
	}

}
