package perpetualbox;

public class Player extends Entity {
	
	boolean isWalking = false;
	int walkDirection = 0;
	int walkDelay = 0;
	boolean isFirstStep = true;
	int inventory = 0;
	
	public Player(Pos inputPos) {
		super(inputPos, 3);
	}
	
	public void startWalking(int direction) {
		if (!isWalking || direction != walkDirection) {
			isWalking = true;
			walkDirection = direction;
			isFirstStep = true;
		}
	}
	
	public void stopWalking(int direction) {
		if (direction == walkDirection) {
			isWalking = false;
			if (walkDelay > 2) {
				walkDelay = 2;
			}
		}
	}
	
	public int getInventory() {
		return inventory;
	}
	
	public void setInventory(int inputInventory) {
		inventory = inputInventory;
	}
	
	public void stepOverObstacle(Pos pos, Pos offset) {
		Pos tempPos = getPos().copy();
		Pos tempPos2 = new Pos(0, 0);
		while (tempPos.getY() > pos.getY() - 2) {
			tempPos.setY(tempPos.getY() - 1);
			tempPos2.set(tempPos);
			if (getWorld().getTerrainData(tempPos2, false) != 0) {
				return;
			}
			tempPos2.setX(tempPos2.getX() + 1);
			if (getWorld().getTerrainData(tempPos2, false) != 0) {
				return;
			}
		}
		if (offset.getX() > 0) {
			tempPos.setX(tempPos.getX() + 2);
		}
		if (offset.getX() < 0) {
			tempPos.setX(tempPos.getX() - 1);
		}
		tempPos2.set(tempPos);
		if (getWorld().getTerrainData(tempPos2, false) != 0) {
			return;
		}
		tempPos2.setY(tempPos2.getY() + 1);
		if (getWorld().getTerrainData(tempPos2, false) != 0) {
			return;
		}
		tempPos.setX(getPos().getX() + offset.getX());
		setPos(tempPos);
	}
	
	public void removeBlock(Pos offset) {
		Pos tempPos = getPos().copy();
		if (offset.getX() > 0) {
			tempPos.setX(tempPos.getX() + 2);
		}
		if (offset.getX() < 0) {
			tempPos.setX(tempPos.getX() - 1);
		}
		if (offset.getY() > 0) {
			tempPos.setY(tempPos.getY() + 2);
		}
		if (offset.getY() < 0) {
			tempPos.setY(tempPos.getY() - 1);
		}
		Pos tempPos2 = getWorld().getBlockPos(tempPos);
		if (tempPos2 != null) {
			int tempData = getWorld().getTerrainData(tempPos2, false);
			if (tempData == 2) {
				getWorld().addBoxUpdatesAbovePos(tempPos2);
				getWorld().removeTerrainBlock(tempPos2);
				inventory = 1;
				return;
			}
		}
		if (offset.getX() != 0) {
			tempPos.setY(tempPos.getY() + 1);
		}
		if (offset.getY() != 0) {
			tempPos.setX(tempPos.getX() + 1);
		}
		tempPos2 = getWorld().getBlockPos(tempPos);
		if (tempPos2 != null) {
			int tempData = getWorld().getTerrainData(tempPos2, false);
			if (tempData == 2) {
				getWorld().addBoxUpdatesAbovePos(tempPos2);
				getWorld().removeTerrainBlock(tempPos2);
				inventory = 1;
				return;
			}
		}
	}
	
	public void placeBlock(Pos offset) {
		Pos tempPos = getPos().copy();
		if (offset.getX() > 0) {
			tempPos.setX(tempPos.getX() + 2);
		}
		if (offset.getX() < 0) {
			tempPos.setX(tempPos.getX() - 2);
		}
		if (offset.getY() > 0) {
			tempPos.setY(tempPos.getY() + 2);
		}
		if (offset.getY() < 0) {
			tempPos.setY(tempPos.getY() - 2);
		}
		if (getWorld().blockIsEmpty(tempPos)) {
			getWorld().addTerrainBlock(tempPos, 2);
			getWorld().addBoxUpdate(tempPos);
			inventory = 0;
		}
	}
	
	public void placeOrRemoveBlock(Pos offset) {
		if (inventory > 0) {
			placeBlock(offset);
		} else {
			removeBlock(offset);
		}
	}
	
	public void tick() {
		super.tick();
		if (walkDelay > 0) {
			walkDelay -= 1;
		}
		if (isWalking) {
			if (walkDelay <= 0) {
				move(new Pos(walkDirection, 0));
				if (isFirstStep) {
					walkDelay = 5;
					isFirstStep = false;
				} else {
					walkDelay = 2;
				}
			}
		}
	}
	
}

