package perpetualbox;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class World {

	public static int CHUNK_UNLOAD_THRESHOLD = 50;
	public static int MAXIMUM_SAVE_DELAY = 1200;
	
	private Player player;
	private List<Chunk> chunkList = new ArrayList<Chunk>();
	private int saveDelay = 0;
	private List<Pos> boxUpdateList = new ArrayList<Pos>();
	private Set<Pos> boxUpdateSet = new HashSet<Pos>();
	
	public World(Player inputPlayer) {
		player = inputPlayer;
		player.setWorld(this);
	}
	
	public Player getPlayer() {
		return player;
	}
	
	public int getChunkCount() {
		return chunkList.size();
	}
	
	public Chunk getChunk(int index) {
		return chunkList.get(index);
	}
	
	private static int roundDown(int number, int multiple) {
		if (number >= 0) {
			return number / multiple * multiple;
		} else {
			return (number - multiple + 1) / multiple * multiple;
		}
	}
	
	public Pos convertPosToChunkPos(Pos pos) {
		return new Pos(roundDown(pos.getX(), Chunk.CHUNK_SIZE),
				roundDown(pos.getY(), Chunk.CHUNK_SIZE));
	}
	
	public Chunk getChunk(Pos pos, boolean shouldCreateChunk) {
		int index = 0;
		while (index < chunkList.size()) {
			Chunk tempChunk = chunkList.get(index);
			if (tempChunk.containsPos(pos)) {
				return tempChunk;
			}
			index += 1;
		}
		if (!shouldCreateChunk) {
			return null;
		}
		Pos tempPos = convertPosToChunkPos(pos);
		Chunk output = Database.loadChunk(tempPos, this);
		chunkList.add(output);
		return output;
	}
	
	public int getTerrainData(Pos pos, boolean shouldCreateChunk) {
		Chunk tempChunk = getChunk(pos, shouldCreateChunk);
		if (tempChunk == null) {
			return 1;
		} else {
			return tempChunk.getTerrainData(pos);
		}
	}
	
	public void setTerrainData(Pos pos, int data) {
		Chunk tempChunk = getChunk(pos, true);
		tempChunk.setTerrainData(pos, data);
	}
	
	public void addTerrainBlock(Pos pos, int data) {
		Pos tempPos = pos.copy();
		setTerrainData(tempPos, data);
		tempPos.setX(tempPos.getX() + 1);
		setTerrainData(tempPos, 1);
		tempPos.setY(tempPos.getY() + 1);
		setTerrainData(tempPos, 1);
		tempPos.setX(tempPos.getX() - 1);
		setTerrainData(tempPos, 1);
	}
	
	public void removeTerrainBlock(Pos pos) {
		Pos tempPos = pos.copy();
		setTerrainData(tempPos, 0);
		tempPos.setX(tempPos.getX() + 1);
		setTerrainData(tempPos, 0);
		tempPos.setY(tempPos.getY() + 1);
		setTerrainData(tempPos, 0);
		tempPos.setX(tempPos.getX() - 1);
		setTerrainData(tempPos, 0);
	}
	
	public boolean blockIsEmpty(Pos pos) {
		Pos tempPos = pos.copy();
		if (getTerrainData(tempPos, false) != 0) {
			return false;
		}
		tempPos.setX(tempPos.getX() + 1);
		if (getTerrainData(tempPos, false) != 0) {
			return false;
		}
		tempPos.setY(tempPos.getY() + 1);
		if (getTerrainData(tempPos, false) != 0) {
			return false;
		}
		tempPos.setX(tempPos.getX() - 1);
		if (getTerrainData(tempPos, false) != 0) {
			return false;
		}
		return true;
	}
	
	public Pos getBlockPos(Pos pos) {
		Pos output = pos.copy();
		if (getTerrainData(output, false) > 1) {
			return output;
		}
		output.setX(output.getX() - 1);
		if (getTerrainData(output, false) > 1) {
			return output;
		}
		output.setY(output.getY() - 1);
		if (getTerrainData(output, false) > 1) {
			return output;
		}
		output.setX(output.getX() + 1);
		if (getTerrainData(output, false) > 1) {
			return output;
		}
		return null;
	}
	
	public void addBoxUpdate(Pos pos) {
		if (!boxUpdateSet.contains(pos)) {
			boxUpdateList.add(pos);
		}
	}
	
	public void addBoxUpdatesAbovePos(Pos pos) {
		Pos tempPos = pos.copy();
		tempPos.setX(tempPos.getX() - 1);
		tempPos.setY(tempPos.getY() - 2);
		while (tempPos.getX() < pos.getX() + 2) {
			Pos tempPos2 = getBlockPos(tempPos);
			if (tempPos2 != null) {
				addBoxUpdate(tempPos2);
				tempPos.setX(tempPos2.getX() + 2);
			}
			tempPos.setX(tempPos.getX() + 1);
			tempPos2 = getBlockPos(tempPos);
			if (tempPos2 != null) {
				addBoxUpdate(tempPos2);
			}
		}
		tempPos = pos.copy();
		tempPos.setX(tempPos.getX() - 1);
		Pos tempPos2 = getBlockPos(tempPos);
		if (tempPos2 != null) {
			addBoxUpdate(tempPos2);
		} else {
			tempPos.setY(tempPos.getY() + 1);
			tempPos2 = getBlockPos(tempPos);
			if (tempPos2 != null) {
				addBoxUpdate(tempPos2);
			}
		}
		tempPos = pos.copy();
		tempPos.setX(tempPos.getX() + 2);
		tempPos2 = getBlockPos(tempPos);
		if (tempPos2 != null) {
			addBoxUpdate(tempPos2);
		} else {
			tempPos.setY(tempPos.getY() + 1);
			tempPos2 = getBlockPos(tempPos);
			if (tempPos2 != null) {
				addBoxUpdate(tempPos2);
			}
		}
	}
	
	public void updateBox(Pos pos) {
		int tempData = getTerrainData(pos, false);
		if (tempData != 2) {
			return;
		}
		Pos tempPos = pos.copy();
		tempPos.setY(tempPos.getY() + 2);
		int tempData1 = getTerrainData(tempPos, false);
		tempPos.setX(tempPos.getX() + 1);
		int tempData2 = getTerrainData(tempPos, false);
		if (tempData1 == 0 && tempData2 == 0) {
			tempPos = pos.copy();
			tempPos.setY(tempPos.getY() + 1);
			addBoxUpdatesAbovePos(pos);
			removeTerrainBlock(pos);
			addTerrainBlock(tempPos, 2);
			addBoxUpdate(tempPos);
			return;
		}
		if (tempData1 == 0) {
			tempPos = pos.copy();
			tempPos.setX(tempPos.getX() - 1);
			int tempData3a = getTerrainData(tempPos, false);
			tempPos.setY(tempPos.getY() + 1);
			int tempData3b = getTerrainData(tempPos, false);
			tempPos.setY(tempPos.getY() + 1);
			int tempData3c = getTerrainData(tempPos, false);
			if (tempData3a == 0 && tempData3b == 0 && tempData3c == 0) {
				tempPos = pos.copy();
				tempPos.setX(tempPos.getX() - 1);
				tempPos.setY(tempPos.getY() + 1);
				addBoxUpdatesAbovePos(pos);
				removeTerrainBlock(pos);
				addTerrainBlock(tempPos, 2);
				addBoxUpdate(tempPos);
				return;
			}
		}
		if (tempData2 == 0) {
			tempPos = pos.copy();
			tempPos.setX(tempPos.getX() + 2);
			int tempData4a = getTerrainData(tempPos, false);
			tempPos.setY(tempPos.getY() + 1);
			int tempData4b = getTerrainData(tempPos, false);
			tempPos.setY(tempPos.getY() + 1);
			int tempData4c = getTerrainData(tempPos, false);
			if (tempData4a == 0 && tempData4b == 0 && tempData4c == 0) {
				tempPos = pos.copy();
				tempPos.setX(tempPos.getX() + 1);
				tempPos.setY(tempPos.getY() + 1);
				addBoxUpdatesAbovePos(pos);
				removeTerrainBlock(pos);
				addTerrainBlock(tempPos, 2);
				addBoxUpdate(tempPos);
				return;
			}
		}
		
	}
	
	public void unloadDistantChunks() {
		int index = chunkList.size() - 1;
		while (index >= 0) {
			Chunk tempChunk = chunkList.get(index);
			int tempPosX1 = player.getPos().getX();
			int tempPosX2 = tempChunk.getPos().getX();
			int tempPosY1 = player.getPos().getY();
			int tempPosY2 = tempChunk.getPos().getY();
			if (tempPosX1 < tempPosX2 - CHUNK_UNLOAD_THRESHOLD || tempPosX1 > tempPosX2 + Chunk.CHUNK_SIZE + CHUNK_UNLOAD_THRESHOLD
					|| tempPosY1 < tempPosY2 - CHUNK_UNLOAD_THRESHOLD || tempPosY1 > tempPosY2 + Chunk.CHUNK_SIZE + CHUNK_UNLOAD_THRESHOLD) {
				chunkList.remove(index);
				Database.persistChunk(tempChunk);
			}
			index -= 1;
		}
	}
	
	public void tick() {
		player.tick();
		List<Pos> tempBoxUpdateList = new ArrayList<Pos>(boxUpdateList);
		boxUpdateList = new ArrayList<Pos>();
		boxUpdateSet = new HashSet<Pos>();
		int index = 0;
		while (index < tempBoxUpdateList.size()) {
			Pos tempPos = tempBoxUpdateList.get(index);
			//System.out.println(tempPos.toString());
			updateBox(tempPos);
			index += 1;
		}
		unloadDistantChunks();
		saveDelay += 1;
		if (saveDelay > MAXIMUM_SAVE_DELAY) {
			Database.persistWorld(this);
			saveDelay = 0;
		}
	}
	
}
